# -*- coding : utf-8 -*-

from . import Kami

__name__ = "kami"
__title__ = "kamilib"
# __version__ = "0.1.2"
__licence__ = "MIT"
__doc__ = "HTR / OCR models evaluation agnostic Python package, originally based on the Kraken transcription system."
